@@ -38,6 +38,8 @@ local tonumber = tonumber
 local math = math
 local getmetatable = getmetatable
 local error = error
+local end_symbol = "..."
+local end_break_symbol = "...\n"
 
 local UNESCAPES = {
   ['0'] = "\x00", z = "\x00", N    = "\x85",
@@ -183,7 +185,14 @@ function Parser:parsestring(line, stopper)
     if not i then
       return nil, line
     end
-    return ssub(line, 2, i-1), ssub(line, i+1)
+    -- Unescape repeated single quotes.
+    while i < #line and ssub(line, i+1, i+1) == "'" do
+      i = sfind(line, "'", i + 2, true)
+      if not i then
+        return nil, line
+      end
+    end
+    return ssub(line, 2, i-1):gsub("''", "'"), ssub(line, i+1)
   end
   if q == '"' then
     local i, buf = 2, ''
@@ -624,8 +633,10 @@ function Parser:parseseq(line, lines, indent)
       error("did not find expected alphabetic or numeric character")
     elseif rest then
       -- Array entry with a value
+      local nextline = lines[1]
+      local indent2 = countindent(nextline)
       tremove(lines, 1)
-      tinsert(seq, self:parsescalar(rest, lines))
+      tinsert(seq, self:parsescalar(rest, lines, indent2))
     end
   end
   return seq
@@ -702,6 +713,13 @@ function Parser:parsemap(line, lines, indent)
   while #lines > 0 do
     -- Check for a new document
     line = lines[1]
+    if line == end_symbol or line == end_break_symbol then
+      for i, _ in ipairs(lines) do
+        lines[i] = nil
+      end
+      return map
+    end
+
     if startswith(line, '---') then
       while #lines > 0 and not startswith(lines, '---') do
         tremove(lines, 1)
